#!/bin/bash
#
# Configure Apache
#    Copyright (C) 2017  AleaJactaEst
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.

usage()
{
cat << EOF
usage:$0 [options]
  prepare mysql (create directory, update configuration)

options:
  -h, --help              : Show this help
  -d, --debug             : Show debug message
EOF
}


#####################
# MAIN
#####################
source /opt/ext/servercontainer_function.sh
msg_info "[$(basename $0):$LINENO] => START"

while test $# -gt 0
do
   case "$1" in
      -h|--help)
         usage
         exit 1
         ;;
      -d|--debug)
         set_debug 1
         shift
         ;;
      *)
         msg_error "options '$1' not recognize"
         usage
         exit 1
         ;;
   esac
done


####################################
# Load Environment
####################################
msg_debug "Load environment"
if [[ ! -f /opt/khanat_config.sh ]]
then
   echo "ERROR - missing /opt/khanat_config.sh"
   exit 2
fi
source /opt/khanat_config.sh

if [[ ! -f /home/gameserver/.bashrc ]]
then
   echo "ERROR - missing /home/gameserver/.bashrc"
   exit 2
fi
source /home/gameserver/.bashrc


#####################
# Install web service
#####################
msg_debug "Install web service"
# create_recursive_link '/home/gameserver/ext/khanatweb' "$KHANAT_PATH/khanatweb"
su -c "cp -r /home/gameserver/ext/khanatweb/* $KHANAT_PATH/khanatweb" gameserver


#####################
# Create apache log
#####################
msg_debug "Create apache log"
create_dir_gameserver '/home/gameserver/log/apache2'

touch /home/gameserver/log/apache2/access.log || exit 2
chown_gameserver '/home/gameserver/log/apache2/access.log' || exit 2

touch /home/gameserver/log/apache2/error.log || exit 2
chown_gameserver '/home/gameserver/log/apache2/error.log' || exit 2

touch /home/gameserver/log/apache2/other_vhosts_access.log || exit 2
chown_gameserver '/home/gameserver/log/apache2/other_vhosts_access.log' || exit 2


#####################
# Start & Stop apache2
#####################
msg_debug "Stop/Start apache"
service apache2 start || exit 2
service apache2 stop || exit 2


#####################
# End
#####################
msg_info "[$(basename $0):$LINENO] => END"

